/*
 * Daia Radu-Petru
 * 332CA
 * LCD Tetris (BETA version)
 */

#include "lcd.h"
#include "usart.h"

#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>


#define START_TEXT_FIRST_ROW 0x03
#define START_TEXT_SECOND_ROW 40

#define N_CHARACTERS 4
#define LINES_PER_CHARACTER 8


//caracterele definite
uint8_t lcd_pieces[N_CHARACTERS][LINES_PER_CHARACTER] = {{ 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00 },
							{ 0x00, 0x00, 0xFE, 0xFE, 0xFE, 0xFE, 0x00, 0x00 },
							{ 0x00, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C, 0x00 },
							{ 0x00, 0x18, 0x18, 0x1E, 0x1E, 0x18, 0x18, 0x00 }};

//starea de inceput
void start_tetris(){
	LCD_writeInstr(LCD_INSTR_clearDisplay);
	LCD_printAt(START_TEXT_FIRST_ROW, "Welcome to");
	LCD_printAt(START_TEXT_SECOND_ROW, "   LCD Tetris!");
}

//inceperea jocului
void wait_start(){		
	
	while((PINB & (1 << PB2)));
	
	int i;
	char buf[15];
	for(i = 3; i >= 0; --i){
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		memset(buf, 0, 15 * sizeof(char));
		sprintf(buf, "Starting in %d", i);
		LCD_printAt(0x00, buf);
		_delay_ms(1000);
	}
	
}

//terminarea jocului
void leaving_game(){
	
	int i;
	char buf[15];
	for(i = 3; i >= 0; --i){
		LCD_writeInstr(LCD_INSTR_clearDisplay);
		memset(buf, 0, 15 * sizeof(char));
		sprintf(buf, "Leaving in %d", i);
		LCD_printAt(0x00, buf);
		_delay_ms(1000);
	}
}

void test_characters(){
	uint8_t pos = 0x00;
	int i, j;
	for(i = 0; i < N_CHARACTERS; ++i){
		 LCD_writeInstr(LCD_INSTR_clearDisplay);
		 LCD_writeInstr(LCD_INSTR_CGRAM);
		 for(j = 0; j < LINES_PER_CHARACTER; ++j){
			LCD_writeData(lcd_pieces[0][j]);
		 }
		 LCD_writeInstr(LCD_INSTR_DDRAM);
		 LCD_putCharAt(pos++, 0);
	}
}

int score;

//afisarea matricei jocului
void display_game_matrix(uint8_t game_matrix[16][2][LINES_PER_CHARACTER])
{
	LCD_writeInstr(LCD_INSTR_clearDisplay);
	uint8_t i, j, k;
	
	for(i = 0; i < 2; ++i){
		uint8_t start = i * 0x40;
		
		
		for(j = 15; j >= 0; --j){
			//creez caracter custom
			 LCD_writeInstr(LCD_INSTR_CGRAM);
			 for(uint8_t k = 0; k < LINES_PER_CHARACTER; ++k)
				LCD_writeData(game_matrix[j][i][k]);
			 LCD_writeInstr(LCD_INSTR_DDRAM);
			 
			LCD_putCharAt(start + j, 0);
		}
	}
}

//tetris
void gameplay(){
	
	score = 0;
	LCD_writeInstr(LCD_INSTR_clearDisplay);
	LCD_print("Gameplay()");
	
	int current_object_y = 0x0F;
	int current_object_x = 0;
	
	uint8_t game_matrix[16][2][LINES_PER_CHARACTER];
	memset(game_matrix, 0, 16 * 2 * LINES_PER_CHARACTER * sizeof(uint8_t));
	
	int occupied_cell[16][2];
	memset(occupied_cell, 0, 16 * 2 * sizeof(int));
	
	//srand(time(NULL));
	int piece_chosen = rand() % N_CHARACTERS;
	
	int last_object_y = 0x0F;
	int last_object_x = 0;
	int choose_new_character = 0;
	
	for(;;){
				
		
		if(choose_new_character){
			last_object_y = current_object_y = 0x0F;
			last_object_x = current_object_x = 0;
			choose_new_character = 0;
			piece_chosen = rand() % N_CHARACTERS;
			memcpy(game_matrix[current_object_y][current_object_x], lcd_pieces[piece_chosen], N_CHARACTERS * sizeof(uint8_t));
			//display_game_matrix(game_matrix);
			_delay_ms(500);
			continue;
		}
		
		//coborarea piesei cu o celula
		if((current_object_y - 1) > 0 && (occupied_cell[current_object_y - 1][current_object_x] == 0)){
			--current_object_y;
		}
		else{
			occupied_cell[current_object_y][current_object_x] = 1;
		}
		
		//mutarea stanga-dreapta
		if((PINB & (1 << PB0)) == 0){
			if(current_object_x > 0){
				--current_object_x;
			}
		}
		else if((PINB & (1 << PB1)) == 0){
			if(current_object_x == 0){
				++current_object_x;
			}
		}
		//stergerea piesei de la pozitia anterioara
		memset(game_matrix[last_object_y][last_object_x], 0, LINES_PER_CHARACTER * sizeof(uint8_t));
		memcpy(game_matrix[current_object_y][current_object_x], lcd_pieces[piece_chosen], N_CHARACTERS * sizeof(uint8_t));
		
		if((PINB & (1 << PB2)) == 0){
			leaving_game();
			//display_score();
			return;
		}
		//display_game_matrix(game_matrix);

		_delay_ms(500);
	}
	
	
}

//afisarea scorului la sfarsitul jocului
void display_score(){
	LCD_writeInstr(LCD_INSTR_clearDisplay);
	char score_text[10];
	memset(score_text, 0, 10 * sizeof(char));
	sprintf(score_text, "Score: %d", score);
	LCD_print(score_text);
}


int main()
{
    LCD_init();
	LCD_writeInstr(LCD_INSTR_displayConfig & ~((1 << 1) | 1));

	DDRB &= ~((1 << PB0) | (1 << PB1) | (1 << PB2));
	PORTB |= ((1 << PB0) | (1 << PB1) | (1 << PB2));


	while(1){
		start_tetris();
		wait_start();
		gameplay();
		display_score();
		while((PINB & (1 << PB2)));
		_delay_ms(500);
	}
	
    return 0;
}

